# Cards Library: Thumbnail

In this page you can find info about:

* [Basic usage](#basic-usage)
* [Thumbnail from Resource ID](#thumbnail-from-resource-id)
* [Thumbnail from Resource URL](#thumbnail-from-resource-url)
* [Error resource id](#error-resource-id)
* [Customize Thumbnail](#customize-thumbnail)
* [Broadcast to know when the download is finished](#broadcast-to-know-when-the-download-is-finished)
* [Using external library](#using-external-library)
* [Using with a Bitmap](#using-with-a-bitmap)


### Basic usage

Creating a base `CardThumbnail` is pretty simple.

``` java
        //Create thumbnail
        CardThumbnail thumb = new CardThumbnail(getActivity().getApplicationContext());

        //Set resource
        thumb.setDrawableResource(R.drawable.carddemo_ic_gmaps_large);

        //Add thumbnail to a card
        card.addCardThumbnail(thumb);
```

`CardThumbnail` provides a base ThumbnailLayout with an ImageView.

You can find it in `res/layout/base_thumbnail_layout.xml`.

With your `CardThumbnail` you can:

* load a image from a resource ID
* load a image from a URL (of course it requires INTERNET  permission)

### Thumbnail from Resource ID

If you want to load an image from a resource ID:

``` java
        //Create a Card
        Card card = new Card(getActivity().getApplicationContext());

        //Create thumbnail
        CardThumbnail thumb = new CardThumbnail(getActivity().getApplicationContext());

        //Set ID resource
        thumb.setDrawableResource(R.drawable.carddemo_ic_gmaps_large);

        //Add thumbnail to a card
        card.addCardThumbnail(thumb);

        //Set card in the cardView
        CardView cardView = (CardView) getActivity().findViewById(R.id.carddemo_thumb_id);
        cardView.setCard(card);
```

![Screen](https://github.com/gabrielemariotti/cardslib/raw/master/demo/images/thumb/resourceId.png)

### Thumbnail from Resource URL

If you want to load an image from an url:

``` java
         //Create a Card
         Card card = new Card(getActivity().getApplicationContext());

        //Create thumbnail
        CardThumbnail thumb = new CardThumbnail(getActivity().getApplicationContext());

        //Set URL resource
        thumb.setUrlResource("https://lh5.googleusercontent.com/-N8bz9q4Kz0I/AAAAAAAAAAI/AAAAAAAAAAs/Icl2bQMyK7c/s265-c-k-no/photo.jpg");

        //Add thumbnail to a card
        card.addCardThumbnail(thumb);

        //Set card in the cardView
        CardView cardView = (CardView) getActivity().findViewById(R.id.carddemo_thumb_url);
        cardView.setCard(card);
```

![Screen](https://github.com/gabrielemariotti/cardslib/raw/master/demo/images/thumb/resourceURL.png)

### Error Resource Id

You can set a drawable resource Id to display when the image can't be attached to ImageView (for example an error occurs while downloading).

``` java
        //Create thumbnail
        CustomThumbCard thumb = new CustomThumbCard(getActivity().getApplicationContext());

        //Set URL resource
        thumb.setUrlResource(myUrl);

        //Error Resource ID
        thumb.setErrorResource(R.drawable.ic_error_loading);

        //Add thumbnail to a card
        card.addCardThumbnail(thumb);
```

![Screen](https://github.com/gabrielemariotti/cardslib/raw/master/demo/images/thumb/errorID.png)

### Customize Thumbnail

If you want to customize your Thumbnail you can use your style files.
The quickest way to start with this would be to copy the specific style or resources in your project and
change them.

``` xml

    <style name="card_thumbnail_image">
        <item name="android:layout_width">@dimen/card_thumbnail_width</item>
        <item name="android:layout_height">@dimen/card_thumbnail_height</item>
        <item name="android:layout_gravity">center</item>
        <item name="android:gravity">center</item>
        <item name="android:scaleType">centerCrop</item>
    </style>

    <style name="card_thumbnail_outer_layout">
        <item name="android:layout_width">wrap_content</item>
        <item name="android:layout_height">wrap_content</item>
    </style>

```

Otherwise you can extend your `CardThumbmail` and override the `setupInnerViewElements` method.

``` java
    public class CustomThumbCard extends CardThumbnail {

        public CustomThumbCard(Context context) {
            super(context);
        }

        @Override
        public void setupInnerViewElements(ViewGroup parent, View viewImage) {
            if (viewImage!=null){
                viewImage.getLayoutParams().width=250;
                viewImage.getLayoutParams().height=250;
            }
        }
    }
```

![Screen](https://github.com/gabrielemariotti/cardslib/raw/master/demo/images/thumb/thumb_style.png)


### Broadcast to know when the download is finished

The `CardThumbnail` loads a Bitmap with a resource ID or with a URL using `LRUCache` and an `AsyncTask`.
If you would like to know when the image is downloaded and attached to ImageView you can create a Broadcast Receiver that listens for
`Constants.IntentManager.INTENT_ACTION_IMAGE_DOWNLOADED` Broadcast Intents, as shown:

``` java
    activity.registerReceiver(mReceiver,new IntentFilter(Constants.IntentManager.INTENT_ACTION_IMAGE_DOWNLOADED));
```

This Intent includes extras that provide additional details:

1. `Constants.IntentManager.INTENT_ACTION_IMAGE_DOWNLOADED_EXTRA_RESULT` is a boolean. It is `true` when the image is downloaded and attached successfully, `false` otherwise.
2. `Constants.IntentManager.INTENT_ACTION_IMAGE_DOWNLOADED_EXTRA_CARD_ID` contains the card id which contains the imageView.
3. `Constants.IntentManager.INTENT_ACTION_IMAGE_DOWNLOADED_EXTRA_ERROR_LOADING` is a boolean. It is `true` when the error image is attached.

``` java
    /**
     * Broadcast for image downloaded by CardThumbnail
     */
    private class ImageBroadcastReceiver extends BroadcastReceiver {

        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle extras = intent.getExtras();
            if (extras!=null){
                boolean result = extras.getBoolean(Constants.IntentManager.INTENT_ACTION_IMAGE_DOWNLOADED_EXTRA_RESULT);
                String id = extras.getString(Constants.IntentManager.INTENT_ACTION_IMAGE_DOWNLOADED_EXTRA_CARD_ID);
                boolean processError = extras.getBoolean(Constants.IntentManager.INTENT_ACTION_IMAGE_DOWNLOADED_EXTRA_ERROR_LOADING);
                if (result){
                    if (id!=null && id.equalsIgnoreCase(cardGmap.getId())){
                        updateIntentToShare();
                    }
                }
            }
        }
    }
```

### Using external library

The `CardThumbnail` loads a Bitmap with a built-in feature which uses `LRUCache` and an `AsyncTask`.

If you want to improve this feature using your favorite networking library you can do this:

First you need to set  `thumbnail.setExternalUsage(true);`

``` java
        Card mCard= new Card(getContext());

        //Add Thumbnail
        MyThumbnail thumbnail = new MyThumbnail(getContext());
        //You need to set true to use an external library
        thumbnail.setExternalUsage(true);
        addCardThumbnail(thumbnail);
```

Then you have to implement your logic in `setupInnerViewElements(ViewGroup parent, View viewImage)` method:

``` java
        public class MyThumbnail extends CardThumbnail {

        @Override
        public void setupInnerViewElements(ViewGroup parent, View viewImage) {

            //Here you have to set your image with an external library
            Picasso.with(getContext())
                   .load(myUrl)
                   .into((ImageView)viewImage);

            viewImage.getLayoutParams().width = 250;
            viewImage.getLayoutParams().height = 250;
        }
```

In Demo/Extra you can find 3 example with Picasso, Ion and Android-Universal-Image-Loader libraries.

You can see `PicassoCard` , `IonCard` , `UniversalImageLoaderCard` sources in demo-extras.

[You can read more info in this page](https://github.com/gabrielemariotti/cardslib/tree/master/doc/OTHERLIBRARIES.md).


### Using with a Bitmap

If you want to use the CardThumbnail with a Bitamp or other DrawableResource you can use the method above:

``` java
        Card mCard= new Card(getContext());

        //Add Thumbnail
        MyThumbnail thumbnail = new MyThumbnail(getContext());
        //You need to set true to use an external library
        thumbnail.setExternalUsage(true);
        addCardThumbnail(thumbnail);

        public class MyThumbnail extends CardThumbnail {

        @Override
        public void setupInnerViewElements(ViewGroup parent, View viewImage) {

            ImageView image= (ImageView)viewImage ;

            //...
        }
```


---

Google and the Google Maps logo are registered trademarks of Google Inc.